Introduction to Semantic Patching of C programs with Coccinelle: Exercises
==========================================================================
# Develop a semantic patch for a complete program

Step-by-step exercises developing a semantic patch for the the `is.c`
benchmark from the NAS Parallel Benchmarks Version 3.4 (NPB3.4).

Exercises originally used at the
 *"Introduction to Semantic Patching of C programs with Coccinelle"*
training given at the Leibniz Supercomputing Centre (LRZ)
in Garching, Germany on October 8th, 2019.

## Coccinelle invocation

Check that Coccinelle is >= 1.0.7:

    $ spatch --version

Check if C file parses correctly:

    $ spatch --parse-c is.c --debug

Check if semantic patch file parses correctly:

    $ spatch --parse-cocci npb_0a_comment_x_EXAMPLE.cocci && echo "OK"

Create diff from a semantic patch file:

    $ spatch is.c --sp-file npb_0a_comment_x_EXAMPLE.cocci
    $ spatch is.c --sp-file npb_0a_comment_x_EXAMPLE.cocci > is.diff

Create diff and apply on input file:

    $ cp is.c is.c.bak # backup copy
    $ spatch is.c --sp-file npb_0a_comment_x_EXAMPLE.cocci | patch is.c
    $ diff is.c.bak is.c | cat # inspect applied patch
    $ mv is.c.bak is.c   # revert original

Use shell function to save typing when in BASH:

    $ function spc() { spatch is.c --sp-file $@; }
    $ spc npb_0a_comment_x_EXAMPLE.cocci

Build the 'is' program:

    $ make is

Clean up directory:

    $ make clean


## Exercises 0.a-0.e: simple warm-up examples

Independent patch files to get acquainted with Coccinelle.
For these, no metavariables are necessary: only dots, plus, minus.
    
    # npb_0a_comment_x_EXAMPLE.cocci
    # npb_0b_change_variable_type_EXAMPLE.cocci
    # npb_0c_change_function_type_EXAMPLE.cocci
    # npb_0d_cast_return_EXAMPLE.cocci
    # npb_0e_modified_variable_EXAMPLE.cocci

Play a bit with these: they will be useful in exercises 1a-1e.

## Exercises 0.i-0.z: advanced warm-up examples

The following use metavariables and inheritance:

    # npb_0f_modified_variable_EXAMPLE.cocci
    # npb_0i_inheritance_EXAMPLE.cocci

Play a bit with these: they will be useful in exercise 1z.

## Exercise 1. No metavariables: dots, plus, minus only

Each of those is an exercise:

    # npb_1a_comment_function_EXERCISE.cocci
    # npb_1b_change_variable_type_EXERCISE.cocci
    # npb_1c_change_function_type_EXERCISE.cocci
    # npb_1d_cast_return_EXERCISE.cocci
    # npb_1e_modified_variable_EXERCISE.cocci

Specify context code explicitly or via `...` .
Specify inserts via `+`.
Specify deletions via `-`.
No more grammar knowledge is necessary.
Refer to Exercise 0 for examples.

Once done, put together elements from the above a,b,c,d,e into:

    # npb_1z_change_type_EXERCISE.cocci

Again, the same basic syntax is necessary.

## Exercise 2. Metavariables and inheritance

Develop the same patch, this time using metavariables and inheritance.
This is more sophisticated: please reuse use metavariables:
'identifier', 'assignment operator', 'expression', 'type'.

Each of those is a sub-exercise:

    # npb_2a_comment_function_EXERCISE.cocci
    # npb_2b_change_variable_type_EXERCISE.cocci
    # npb_2c_change_function_type_EXERCISE.cocci
    # npb_2d_cast_return_EXERCISE.cocci
    # npb_2e_modified_variable_EXERCISE.cocci

Once done, put together the rule from a,b,c,d,e, reusing metavariables
in dependencies across rules:

    # npb_2z_change_type_EXERCISE.cocci

Keep in mind that there is more than one way to solve this.


## Licensing

The following source files:

    npbparams.h c_print_results.c c_timers.h c_timers.c wtime.h wtime.c is.c

are originally from NPB3.4 and are necessary to build the 'is' program.
These files are redistributed under their original copyright terms, the
 NASA Open Source Agreement v1.3 license; see
 https://www.nas.nasa.gov/publications/npb.html
and
 https://opensource.org/licenses/nasa1.3.php .
The copy of is.c in this repository has been modified from the original:
line
 `#include "../common/c_timers.h`
has been replaced with
 `#include "c_timers.h"`
by Michele Martone in September 2019.

All the remaining files have been authored and (C) Michele Martone 2019 
and are distributed under the
  Creative Commons Attribution 4.0 International (CC BY 4.0) 
license; see https://creativecommons.org/licenses/by/4.0/deed.en

